--[[ *** LOAD LIBRARIES *** ]]--
local Guilder = _G.Guilder
if not Guilder then
	error("Guilder_CrowdControl requires Guilder")
end

local Guilder_CrowdControl = Guilder:NewModule("Crowd Control", "AceConsole-3.0", "AceEvent-3.0")
local L = Guilder.L
Guilder_CrowdControl.modName = L["Crowd Control"]

--[[ *** SET UP VARIABLES *** ]]--

-- make global API calls local to speed things up
local GetNumPartyMembers = _G.GetNumPartyMembers
local GetNumRaidMembers = _G.GetNumRaidMembers

--[[ *** DEFAULT SETTINGS *** ]]--

local defaults = {
	profile = {
		channel = "SELF",
		RT1Name = "", -- Skull
		RT1Type = "None",
		RT2Name = "", -- Cross
		RT2Type = "None",
		RT3Name = "", -- Square
		RT3Type = "None",
		RT4Name = "", -- Moon
		RT4Type = "None",
		RT5Name = "", -- Triangle
		RT5Type = "None",
		RT6Name = "", -- Diamond
		RT6Type = "None",
		RT7Name = "", -- Circle
		RT7Type = "None",
		RT8Name = "", -- Star
		RT8Type = "None",
	}
}

--[[ *** CONFIGURATION MENU *** ]]--

local options = {
	RT1 = { -- Skull
		type = "group",
		name = L["Skull"],
		desc = L["Setup responsibilities for the Skull target"],
		order = 21,
		args = {
			name = {
				type = "input",
				name = L["Select Name"],
				desc = L["Select the name of the character responsible for this target"],
				order = 1,
				get = function() return Guilder_CrowdControl.db.profile.RT1Name end,
				set = function(info, v) Guilder_CrowdControl.db.profile.RT1Name = v end,
			},
			ttype = {
				type = "select",
				name = L["Select Type"],
				desc = L["Select the type of crowd control to be used on this target"],
				order = 2,
				values = {
					["Banish"] = L["Banish"], ["Cyclone"] = L["Cyclone"], ["Enslave"] = L["Enslave"],
					["Fear"] = L["Fear"], ["Hibernate"] = L["Hibernate"], ["MT"] = L["MT"],
					["MindControl"] = L["Mind Control"], ["None"] = L["None"], ["OT"] = L["OT"],
					["Root"] = L["Root"], ["Sap"] = L["Sap"], ["Seduce"] = L["Seduce"],
					["Shackle"] = L["Shackle"], ["Sheep"] = L["Sheep"], ["Trap"] = L["Trap"],
					["TurnEvil"] = L["Turn Evil"],
				},
				get = function() return Guilder_CrowdControl.db.profile.RT1Type end,
				set = function(info, v) Guilder_CrowdControl.db.profile.RT1Type = v end,
			},
			clear = {
				type = "execute",
				name = L["Clear Name"],
				desc = L["Clear this target of its assigned character and crowd control type"],
				order = 3,
				func = function()
					Guilder_CrowdControl.db.profile.RT1Type = L["None"]
					Guilder_CrowdControl.db.profile.RT1Name = L["EMPTY_STRING"]
				end,
			},
		},
	},
	RT2 = { -- Cross
		type = "group",
		name = L["Cross"],
		desc = L["Setup responsibilities for the Cross target"],
		order = 22,
		args = {
			name = {
				type = "input",
				name = L["Select Name"],
				desc = L["Select the name of the character responsible for this target"],
				order = 1,
				get = function() return Guilder_CrowdControl.db.profile.RT2Name end,
				set = function(info, v) Guilder_CrowdControl.db.profile.RT2Name = v end,
			},
			ttype = {
				type = "select",
				name = L["Select Type"],
				desc = L["Select the type of crowd control to be used on this target"],
				order = 2,
				values = {
					["Banish"] = L["Banish"], ["Cyclone"] = L["Cyclone"], ["Enslave"] = L["Enslave"],
					["Fear"] = L["Fear"], ["Hibernate"] = L["Hibernate"], ["MT"] = L["MT"],
					["MindControl"] = L["Mind Control"], ["None"] = L["None"], ["OT"] = L["OT"],
					["Root"] = L["Root"], ["Sap"] = L["Sap"], ["Seduce"] = L["Seduce"],
					["Shackle"] = L["Shackle"], ["Sheep"] = L["Sheep"], ["Trap"] = L["Trap"],
					["TurnEvil"] = L["Turn Evil"],
				},
				get = function() return Guilder_CrowdControl.db.profile.RT2Type end,
				set = function(info, v) Guilder_CrowdControl.db.profile.RT2Type = v end,
			},
			clear = {
				type = "execute",
				name = L["Clear Name"],
				desc = L["Clear this target of its assigned character and crowd control type"],
				order = 3,
				func = function()
					Guilder_CrowdControl.db.profile.RT2Type = L["None"]
					Guilder_CrowdControl.db.profile.RT2Name = L["EMPTY_STRING"]
				end,
			},
		},
	},
	RT3 = { -- Square
		type = "group",
		name = L["Square"],
		desc = L["Setup responsibilities for the Square target"],
		order = 23,
		args = {
			name = {
				type = "input",
				name = L["Select Name"],
				desc = L["Select the name of the character responsible for this target"],
				order = 1,
				get = function() return Guilder_CrowdControl.db.profile.RT3Name end,
				set = function(info, v) Guilder_CrowdControl.db.profile.RT3Name = v end,
			},
			ttype = {
				type = "select",
				name = L["Select Type"],
				desc = L["Select the type of crowd control to be used on this target"],
				order = 2,
				values = {
					["Banish"] = L["Banish"], ["Cyclone"] = L["Cyclone"], ["Enslave"] = L["Enslave"],
					["Fear"] = L["Fear"], ["Hibernate"] = L["Hibernate"], ["MT"] = L["MT"],
					["MindControl"] = L["Mind Control"], ["None"] = L["None"], ["OT"] = L["OT"],
					["Root"] = L["Root"], ["Sap"] = L["Sap"], ["Seduce"] = L["Seduce"],
					["Shackle"] = L["Shackle"], ["Sheep"] = L["Sheep"], ["Trap"] = L["Trap"],
					["TurnEvil"] = L["Turn Evil"],
				},
				get = function() return Guilder_CrowdControl.db.profile.RT3Type end,
				set = function(info, v) Guilder_CrowdControl.db.profile.RT3Type = v end,
			},
			clear = {
				type = "execute",
				name = L["Clear Name"],
				desc = L["Clear this target of its assigned character and crowd control type"],
				order = 3,
				func = function()
					Guilder_CrowdControl.db.profile.RT3Type = L["None"]
					Guilder_CrowdControl.db.profile.RT3Name = L["EMPTY_STRING"]
				end,
			},
		},
	},
	RT4 = { -- Moon
		type = "group",
		name = L["Moon"],
		desc = L["Setup responsibilities for the Moon target"],
		order = 24,
		args = {
			name = {
				type = "input",
				name = L["Select Name"],
				desc = L["Select the name of the character responsible for this target"],
				order = 1,
				get = function() return Guilder_CrowdControl.db.profile.RT4Name end,
				set = function(info, v) Guilder_CrowdControl.db.profile.RT4Name = v end,
			},
			ttype = {
				type = "select",
				name = L["Select Type"],
				desc = L["Select the type of crowd control to be used on this target"],
				order = 2,
				values = {
					["Banish"] = L["Banish"], ["Cyclone"] = L["Cyclone"], ["Enslave"] = L["Enslave"],
					["Fear"] = L["Fear"], ["Hibernate"] = L["Hibernate"], ["MT"] = L["MT"],
					["MindControl"] = L["Mind Control"], ["None"] = L["None"], ["OT"] = L["OT"],
					["Root"] = L["Root"], ["Sap"] = L["Sap"], ["Seduce"] = L["Seduce"],
					["Shackle"] = L["Shackle"], ["Sheep"] = L["Sheep"], ["Trap"] = L["Trap"],
					["TurnEvil"] = L["Turn Evil"],
				},
				get = function() return Guilder_CrowdControl.db.profile.RT4Type end,
				set = function(info, v) Guilder_CrowdControl.db.profile.RT4Type = v end,
			},
			clear = {
				type = "execute",
				name = L["Clear Name"],
				desc = L["Clear this target of its assigned character and crowd control type"],
				order = 3,
				func = function()
					Guilder_CrowdControl.db.profile.RT4Type = L["None"]
					Guilder_CrowdControl.db.profile.RT4Name = L["EMPTY_STRING"]
				end,
			},
		},
	},
	RT5 = { -- Triangle
		type = "group",
		name = L["Triangle"],
		desc = L["Setup responsibilities for the Triangle target"],
		order = 25,
		args = {
			name = {
				type = "input",
				name = L["Select Name"],
				desc = L["Select the name of the character responsible for this target"],
				order = 1,
				get = function() return Guilder_CrowdControl.db.profile.RT5Name end,
				set = function(info, v) Guilder_CrowdControl.db.profile.RT5Name = v end,
			},
			ttype = {
				type = "select",
				name = L["Select Type"],
				desc = L["Select the type of crowd control to be used on this target"],
				order = 2,
				values = {
					["Banish"] = L["Banish"], ["Cyclone"] = L["Cyclone"], ["Enslave"] = L["Enslave"],
					["Fear"] = L["Fear"], ["Hibernate"] = L["Hibernate"], ["MT"] = L["MT"],
					["MindControl"] = L["Mind Control"], ["None"] = L["None"], ["OT"] = L["OT"],
					["Root"] = L["Root"], ["Sap"] = L["Sap"], ["Seduce"] = L["Seduce"],
					["Shackle"] = L["Shackle"], ["Sheep"] = L["Sheep"], ["Trap"] = L["Trap"],
					["TurnEvil"] = L["Turn Evil"],
				},
				get = function() return Guilder_CrowdControl.db.profile.RT5Type end,
				set = function(info, v) Guilder_CrowdControl.db.profile.RT5Type = v end,
			},
			clear = {
				type = "execute",
				name = L["Clear Name"],
				desc = L["Clear this target of its assigned character and crowd control type"],
				order = 3,
				func = function()
					Guilder_CrowdControl.db.profile.RT5Type = L["None"]
					Guilder_CrowdControl.db.profile.RT5Name = L["EMPTY_STRING"]
				end,
			},
		},
	},
	RT6 = { -- Diamond
		type = "group",
		name = L["Diamond"],
		desc = L["Setup responsibilities for the Diamond target"],
		order = 26,
		args = {
			name = {
				type = "input",
				name = L["Select Name"],
				desc = L["Select the name of the character responsible for this target"],
				order = 1,
				get = function() return Guilder_CrowdControl.db.profile.RT6Name end,
				set = function(info, v) Guilder_CrowdControl.db.profile.RT6Name = v end,
			},
			ttype = {
				type = "select",
				name = L["Select Type"],
				desc = L["Select the type of crowd control to be used on this target"],
				order = 2,
				values = {
					["Banish"] = L["Banish"], ["Cyclone"] = L["Cyclone"], ["Enslave"] = L["Enslave"],
					["Fear"] = L["Fear"], ["Hibernate"] = L["Hibernate"], ["MT"] = L["MT"],
					["MindControl"] = L["Mind Control"], ["None"] = L["None"], ["OT"] = L["OT"],
					["Root"] = L["Root"], ["Sap"] = L["Sap"], ["Seduce"] = L["Seduce"],
					["Shackle"] = L["Shackle"], ["Sheep"] = L["Sheep"], ["Trap"] = L["Trap"],
					["TurnEvil"] = L["Turn Evil"],
				},
				get = function() return Guilder_CrowdControl.db.profile.RT6Type end,
				set = function(info, v) Guilder_CrowdControl.db.profile.RT6Type = v end,
			},
			clear = {
				type = "execute",
				name = L["Clear Name"],
				desc = L["Clear this target of its assigned character and crowd control type"],
				order = 3,
				func = function()
					Guilder_CrowdControl.db.profile.RT6Type = L["None"]
					Guilder_CrowdControl.db.profile.RT6Name = L["EMPTY_STRING"]
				end,
			},
		},
	},
	RT7 = { -- Circle
		type = "group",
		name = L["Circle"],
		desc = L["Setup responsibilities for the Circle target"],
		order = 27,
		args = {
			name = {
				type = "input",
				name = L["Select Name"],
				desc = L["Select the name of the character responsible for this target"],
				order = 1,
				get = function() return Guilder_CrowdControl.db.profile.RT7Name end,
				set = function(info, v) Guilder_CrowdControl.db.profile.RT7Name = v end,
			},
			ttype = {
				type = "select",
				name = L["Select Type"],
				desc = L["Select the type of crowd control to be used on this target"],
				order = 2,
				values = {
					["Banish"] = L["Banish"], ["Cyclone"] = L["Cyclone"], ["Enslave"] = L["Enslave"],
					["Fear"] = L["Fear"], ["Hibernate"] = L["Hibernate"], ["MT"] = L["MT"],
					["MindControl"] = L["Mind Control"], ["None"] = L["None"], ["OT"] = L["OT"],
					["Root"] = L["Root"], ["Sap"] = L["Sap"], ["Seduce"] = L["Seduce"],
					["Shackle"] = L["Shackle"], ["Sheep"] = L["Sheep"], ["Trap"] = L["Trap"],
					["TurnEvil"] = L["Turn Evil"],
				},
				get = function() return Guilder_CrowdControl.db.profile.RT7Type end,
				set = function(info, v) Guilder_CrowdControl.db.profile.RT7Type = v end,
			},
			clear = {
				type = "execute",
				name = L["Clear Name"],
				desc = L["Clear this target of its assigned character and crowd control type"],
				order = 3,
				func = function()
					Guilder_CrowdControl.db.profile.RT7Type = L["None"]
					Guilder_CrowdControl.db.profile.RT7Name = L["EMPTY_STRING"]
				end,
			},
		},
	},
	RT8 = { -- Star
		type = "group",
		name = L["Star"],
		desc = L["Setup responsibilities for the Star target"],
		order = 28,
		args = {
			name = {
				type = "input",
				name = L["Select Name"],
				desc = L["Select the name of the character responsible for this target"],
				order = 1,
				get = function() return Guilder_CrowdControl.db.profile.RT8Name end,
				set = function(info, v) Guilder_CrowdControl.db.profile.RT8Name = v end,
			},
			ttype = {
				type = "select",
				name = L["Select Type"],
				desc = L["Select the type of crowd control to be used on this target"],
				order = 2,
				values = {
					["Banish"] = L["Banish"], ["Cyclone"] = L["Cyclone"], ["Enslave"] = L["Enslave"],
					["Fear"] = L["Fear"], ["Hibernate"] = L["Hibernate"], ["MT"] = L["MT"],
					["MindControl"] = L["Mind Control"], ["None"] = L["None"], ["OT"] = L["OT"],
					["Root"] = L["Root"], ["Sap"] = L["Sap"], ["Seduce"] = L["Seduce"],
					["Shackle"] = L["Shackle"], ["Sheep"] = L["Sheep"], ["Trap"] = L["Trap"],
					["TurnEvil"] = L["Turn Evil"],
				},
				get = function() return Guilder_CrowdControl.db.profile.RT8Type end,
				set = function(info, v) Guilder_CrowdControl.db.profile.RT8Type = v end,
			},
			clear = {
				type = "execute",
				name = L["Clear Name"],
				desc = L["Clear this target of its assigned character and crowd control type"],
				order = 3,
				func = function()
					Guilder_CrowdControl.db.profile.RT8Type = L["None"]
					Guilder_CrowdControl.db.profile.RT8Name = L["EMPTY_STRING"]
				end,
			},
		},
	},
	Channel = {
		type = "select",
		name = L["Select Channel"],
		desc = L["Select the output channel for the crowd control announcement"],
		values = { ["RAID"] = L["Raid"], ["PARTY"] = L["Party"], ["OFFICER"] = L["Officer"], ["GUILD"] = L["Guild"], ["SAY"] = L["Say"], ["SELF"] = L["Self"] },
		order = 101,
		get = function() return Guilder_CrowdControl.db.profile.channel end,
		set = function(info, v) Guilder_CrowdControl.db.profile.channel = v end,
	},
	Announce = {
		type = "execute",
		name = L["Announce"],
		desc = L["Announce the crowd control targets to your current raid"],
		order = 102,
		func = function() Guilder_CrowdControl:CCTargets(Guilder_CrowdControl.db.profile.channel, nil) end,
	},
	ClearAll = {
		type = "execute",
		name = L["Clear All"],
		desc = L["Clear all targets of their assigned character and crowd control type"],
		order = 103,
		func = function()
			Guilder_CrowdControl.db.profile.RT1Type = L["None"]
			Guilder_CrowdControl.db.profile.RT1Name = L["EMPTY_STRING"]
			Guilder_CrowdControl.db.profile.RT2Type = L["None"]
			Guilder_CrowdControl.db.profile.RT2Name = L["EMPTY_STRING"]
			Guilder_CrowdControl.db.profile.RT3Type = L["None"]
			Guilder_CrowdControl.db.profile.RT3Name = L["EMPTY_STRING"]
			Guilder_CrowdControl.db.profile.RT4Type = L["None"]
			Guilder_CrowdControl.db.profile.RT4Name = L["EMPTY_STRING"]
			Guilder_CrowdControl.db.profile.RT5Type = L["None"]
			Guilder_CrowdControl.db.profile.RT5Name = L["EMPTY_STRING"]
			Guilder_CrowdControl.db.profile.RT6Type = L["None"]
			Guilder_CrowdControl.db.profile.RT6Name = L["EMPTY_STRING"]
			Guilder_CrowdControl.db.profile.RT7Type = L["None"]
			Guilder_CrowdControl.db.profile.RT7Name = L["EMPTY_STRING"]
			Guilder_CrowdControl.db.profile.RT8Type = L["None"]
			Guilder_CrowdControl.db.profile.RT8Name = L["EMPTY_STRING"]
		end,
	},
	ScanParty = {
		type = "execute",
		name = L["Scan Players"],
		desc = L["Rescan the player list"],
		order = 104,
		func = function() Guilder_CrowdControl:Scan() end,
	}
}

--[[ *** MODULE FUNCTIONS *** ]]--

function Guilder_CrowdControl:OnInitialize()
	local db = Guilder.db:RegisterNamespace("Crowd Control", defaults)
	self.db = db
end

function Guilder_CrowdControl:OnEnable()
end

function Guilder_CrowdControl:OnDisable()
end

function Guilder_CrowdControl:Info()
	return L["Select targets for crowd-control responsibilities with this tool"]
end

function Guilder_CrowdControl:GetOptions()
	return options
end

--[[ *** GENERIC FUNCTIONS *** ]]--

local function IsInParty()
	return GetNumPartyMembers() > 0
end

local function IsInRaid()
	return GetNumRaidMembers() > 0
end

local function GetClassHexColor(name)
	local _, class = UnitClass(name)
	if not class then return 'cccccc' end
	local c = RAID_CLASS_COLORS[class]
	return ("%02X%02X%02X"):format(c.r*255, c.g*255, c.b*255)
end

--[[ *** CATCH EVENTS *** ]]--

--- This function scans the party/raid and updates the selection if needed, otherwise resets to input
-- @param nil
-- @usage Guilder_CrowdControl::Scan()
-- @returns nil
function Guilder_CrowdControl:Scan(scanType)
	local groupMembers = nil
	if IsInRaid() then
		-- use raid list
		groupMembers = {}
		for name in LR:GetIterator("NAME", true) do
			groupMembers[name] = "|cFF"..LR:GetClassHexColor(name)..name
		end
	elseif IsInParty() then
		-- use party list
--[[
		for name in LP:GetIterator("NAME", true) do
			groupMembers[name] = "|cFF"..LP:GetClassHexColor(name)..name
		end
]]
		-- scan player first, since Party only give all other members
		groupMembers = {}
		local unitName, _ = UnitName("player")
		groupMembers[unitName] = "|cFF"..GetClassHexColor("player")..unitName
		-- add the rest of the party members
		for i = 1, GetNumPartyMembers() do
			unitName, _ = UnitName("party"..i)
			groupMembers[unitName] = "|cFF"..GetClassHexColor("party"..i)..unitName
		end
	end
	if groupMembers == nil then
		for i = 1, 8 do
			Guilder.ModuleOptions["Crowd_Control"].args["RT"..i].args.name.type = "input"
			Guilder.ModuleOptions["Crowd_Control"].args["RT"..i].args.name.values = nil
		end
	else
		for i = 1, 8 do
			Guilder.ModuleOptions["Crowd_Control"].args["RT"..i].args.name.type = "select"
			Guilder.ModuleOptions["Crowd_Control"].args["RT"..i].args.name.values = groupMembers
		end
	end
end

--[[ *** CROWD CONTROL FUNCTIONS *** ]]--

--- This function announces the crowd control target responsibilities
-- @param channelName name of the channel to send the information to
-- @param recipient person who will receive the whisper
-- @usage Guilder_CrowdControl:CCTargets("RAID")
-- @usage Guilder_CrowdControl:CCTargets("WHISPER", "Player")
-- @returns nil
function Guilder_CrowdControl:CCTargets(channelName, recipient)
	if recipient == nil then
		recipient = ""
	end
	countNumberSetup = 0
	Guilder:SendMessage(L["Crowd Control Responsibilities"], channelName, recipient)
	if Guilder_CrowdControl.db.profile.RT1Type ~= L["None"] then
		Guilder:SendMessage(L["{skull}"].." - "..Guilder_CrowdControl.db.profile.RT1Type.." - "..Guilder_CrowdControl.db.profile.RT1Name, channelName, recipient)
		countNumberSetup = countNumberSetup + 1
	end
	if Guilder_CrowdControl.db.profile.RT2Type ~= L["None"] then
		Guilder:SendMessage(L["{cross}"].." - "..Guilder_CrowdControl.db.profile.RT2Type.." - "..Guilder_CrowdControl.db.profile.RT2Name, channelName, recipient)
		countNumberSetup = countNumberSetup + 1
	end
	if Guilder_CrowdControl.db.profile.RT3Type ~= L["None"] then
		Guilder:SendMessage(L["{square}"].." - "..Guilder_CrowdControl.db.profile.RT3Type.." - "..Guilder_CrowdControl.db.profile.RT3Name, channelName, recipient)
		countNumberSetup = countNumberSetup + 1
	end
	if Guilder_CrowdControl.db.profile.RT4Type ~= L["None"] then
		Guilder:SendMessage(L["{moon}"].." - "..Guilder_CrowdControl.db.profile.RT4Type.." - "..Guilder_CrowdControl.db.profile.RT4Name, channelName, recipient)
		countNumberSetup = countNumberSetup + 1
	end
	if Guilder_CrowdControl.db.profile.RT5Type ~= L["None"] then
		Guilder:SendMessage(L["{triangle}"].." - "..Guilder_CrowdControl.db.profile.RT5Type.." - "..Guilder_CrowdControl.db.profile.RT5Name, channelName, recipient)
		countNumberSetup = countNumberSetup + 1
	end
	if Guilder_CrowdControl.db.profile.RT6Type ~= L["None"] then
		Guilder:SendMessage(L["{diamond}"].." - "..Guilder_CrowdControl.db.profile.RT6Type.." - "..Guilder_CrowdControl.db.profile.RT6Name, channelName, recipient)
		countNumberSetup = countNumberSetup + 1
	end
	if Guilder_CrowdControl.db.profile.RT7Type ~= L["None"] then
		Guilder:SendMessage(L["{circle}"].." - "..Guilder_CrowdControl.db.profile.RT7Type.." - "..Guilder_CrowdControl.db.profile.RT7Name, channelName, recipient)
		countNumberSetup = countNumberSetup + 1
	end
	if Guilder_CrowdControl.db.profile.RT8Type ~= L["None"] then
		Guilder:SendMessage(L["{star}"].." - "..Guilder_CrowdControl.db.profile.RT8Type.." - "..Guilder_CrowdControl.db.profile.RT8Name, channelName, recipient)
		countNumberSetup = countNumberSetup + 1
	end
	if countNumberSetup == 0 then
		Guilder:SendMessage(L["No targets set up, expect imminent chaos!"], channelName, recipient)
	end
end
